# Generated from java-escape by ANTLR 4.5
from antlr4 import *

# This class defines a complete listener for a parse tree produced by blendedParser.
class blendedParserListener(ParseTreeListener):

    # Enter a parse tree produced by blendedParser#start.
    def enterStart(self, ctx):
        pass

    # Exit a parse tree produced by blendedParser#start.
    def exitStart(self, ctx):
        pass


    # Enter a parse tree produced by blendedParser#blendedBlock.
    def enterBlendedBlock(self, ctx):
        pass

    # Exit a parse tree produced by blendedParser#blendedBlock.
    def exitBlendedBlock(self, ctx):
        pass


    # Enter a parse tree produced by blendedParser#blendedBlockParent.
    def enterBlendedBlockParent(self, ctx):
        pass

    # Exit a parse tree produced by blendedParser#blendedBlockParent.
    def exitBlendedBlockParent(self, ctx):
        pass


    # Enter a parse tree produced by blendedParser#blended.
    def enterBlended(self, ctx):
        pass

    # Exit a parse tree produced by blendedParser#blended.
    def exitBlended(self, ctx):
        pass


    # Enter a parse tree produced by blendedParser#blendedNoSet.
    def enterBlendedNoSet(self, ctx):
        pass

    # Exit a parse tree produced by blendedParser#blendedNoSet.
    def exitBlendedNoSet(self, ctx):
        pass


    # Enter a parse tree produced by blendedParser#blockStatementWithParent.
    def enterBlockStatementWithParent(self, ctx):
        pass

    # Exit a parse tree produced by blendedParser#blockStatementWithParent.
    def exitBlockStatementWithParent(self, ctx):
        pass


    # Enter a parse tree produced by blendedParser#blockStatement.
    def enterBlockStatement(self, ctx):
        pass

    # Exit a parse tree produced by blendedParser#blockStatement.
    def exitBlockStatement(self, ctx):
        pass


    # Enter a parse tree produced by blendedParser#extendsStatement.
    def enterExtendsStatement(self, ctx):
        pass

    # Exit a parse tree produced by blendedParser#extendsStatement.
    def exitExtendsStatement(self, ctx):
        pass


    # Enter a parse tree produced by blendedParser#commentStatement.
    def enterCommentStatement(self, ctx):
        pass

    # Exit a parse tree produced by blendedParser#commentStatement.
    def exitCommentStatement(self, ctx):
        pass


    # Enter a parse tree produced by blendedParser#comStatement.
    def enterComStatement(self, ctx):
        pass

    # Exit a parse tree produced by blendedParser#comStatement.
    def exitComStatement(self, ctx):
        pass


    # Enter a parse tree produced by blendedParser#rawStatement.
    def enterRawStatement(self, ctx):
        pass

    # Exit a parse tree produced by blendedParser#rawStatement.
    def exitRawStatement(self, ctx):
        pass


    # Enter a parse tree produced by blendedParser#includeStatement.
    def enterIncludeStatement(self, ctx):
        pass

    # Exit a parse tree produced by blendedParser#includeStatement.
    def exitIncludeStatement(self, ctx):
        pass


    # Enter a parse tree produced by blendedParser#setStatement.
    def enterSetStatement(self, ctx):
        pass

    # Exit a parse tree produced by blendedParser#setStatement.
    def exitSetStatement(self, ctx):
        pass


    # Enter a parse tree produced by blendedParser#macroStatement.
    def enterMacroStatement(self, ctx):
        pass

    # Exit a parse tree produced by blendedParser#macroStatement.
    def exitMacroStatement(self, ctx):
        pass


    # Enter a parse tree produced by blendedParser#ifStatement.
    def enterIfStatement(self, ctx):
        pass

    # Exit a parse tree produced by blendedParser#ifStatement.
    def exitIfStatement(self, ctx):
        pass


    # Enter a parse tree produced by blendedParser#printStatement.
    def enterPrintStatement(self, ctx):
        pass

    # Exit a parse tree produced by blendedParser#printStatement.
    def exitPrintStatement(self, ctx):
        pass


    # Enter a parse tree produced by blendedParser#forStatement.
    def enterForStatement(self, ctx):
        pass

    # Exit a parse tree produced by blendedParser#forStatement.
    def exitForStatement(self, ctx):
        pass


    # Enter a parse tree produced by blendedParser#doStatement.
    def enterDoStatement(self, ctx):
        pass

    # Exit a parse tree produced by blendedParser#doStatement.
    def exitDoStatement(self, ctx):
        pass


    # Enter a parse tree produced by blendedParser#renderStatement.
    def enterRenderStatement(self, ctx):
        pass

    # Exit a parse tree produced by blendedParser#renderStatement.
    def exitRenderStatement(self, ctx):
        pass


    # Enter a parse tree produced by blendedParser#forTag.
    def enterForTag(self, ctx):
        pass

    # Exit a parse tree produced by blendedParser#forTag.
    def exitForTag(self, ctx):
        pass


    # Enter a parse tree produced by blendedParser#forItemsTag.
    def enterForItemsTag(self, ctx):
        pass

    # Exit a parse tree produced by blendedParser#forItemsTag.
    def exitForItemsTag(self, ctx):
        pass


    # Enter a parse tree produced by blendedParser#endForTag.
    def enterEndForTag(self, ctx):
        pass

    # Exit a parse tree produced by blendedParser#endForTag.
    def exitEndForTag(self, ctx):
        pass


    # Enter a parse tree produced by blendedParser#blockTag.
    def enterBlockTag(self, ctx):
        pass

    # Exit a parse tree produced by blendedParser#blockTag.
    def exitBlockTag(self, ctx):
        pass


    # Enter a parse tree produced by blendedParser#endblockTag.
    def enterEndblockTag(self, ctx):
        pass

    # Exit a parse tree produced by blendedParser#endblockTag.
    def exitEndblockTag(self, ctx):
        pass


    # Enter a parse tree produced by blendedParser#parentTag.
    def enterParentTag(self, ctx):
        pass

    # Exit a parse tree produced by blendedParser#parentTag.
    def exitParentTag(self, ctx):
        pass


    # Enter a parse tree produced by blendedParser#macroTag.
    def enterMacroTag(self, ctx):
        pass

    # Exit a parse tree produced by blendedParser#macroTag.
    def exitMacroTag(self, ctx):
        pass


    # Enter a parse tree produced by blendedParser#endmacroTag.
    def enterEndmacroTag(self, ctx):
        pass

    # Exit a parse tree produced by blendedParser#endmacroTag.
    def exitEndmacroTag(self, ctx):
        pass


    # Enter a parse tree produced by blendedParser#ifTag.
    def enterIfTag(self, ctx):
        pass

    # Exit a parse tree produced by blendedParser#ifTag.
    def exitIfTag(self, ctx):
        pass


    # Enter a parse tree produced by blendedParser#elseTag.
    def enterElseTag(self, ctx):
        pass

    # Exit a parse tree produced by blendedParser#elseTag.
    def exitElseTag(self, ctx):
        pass


    # Enter a parse tree produced by blendedParser#endifTag.
    def enterEndifTag(self, ctx):
        pass

    # Exit a parse tree produced by blendedParser#endifTag.
    def exitEndifTag(self, ctx):
        pass


    # Enter a parse tree produced by blendedParser#commentTag.
    def enterCommentTag(self, ctx):
        pass

    # Exit a parse tree produced by blendedParser#commentTag.
    def exitCommentTag(self, ctx):
        pass


    # Enter a parse tree produced by blendedParser#endcommentTag.
    def enterEndcommentTag(self, ctx):
        pass

    # Exit a parse tree produced by blendedParser#endcommentTag.
    def exitEndcommentTag(self, ctx):
        pass


    # Enter a parse tree produced by blendedParser#pathString.
    def enterPathString(self, ctx):
        pass

    # Exit a parse tree produced by blendedParser#pathString.
    def exitPathString(self, ctx):
        pass


    # Enter a parse tree produced by blendedParser#path.
    def enterPath(self, ctx):
        pass

    # Exit a parse tree produced by blendedParser#path.
    def exitPath(self, ctx):
        pass


    # Enter a parse tree produced by blendedParser#setVar.
    def enterSetVar(self, ctx):
        pass

    # Exit a parse tree produced by blendedParser#setVar.
    def exitSetVar(self, ctx):
        pass


    # Enter a parse tree produced by blendedParser#expression.
    def enterExpression(self, ctx):
        pass

    # Exit a parse tree produced by blendedParser#expression.
    def exitExpression(self, ctx):
        pass


    # Enter a parse tree produced by blendedParser#attributeVariable.
    def enterAttributeVariable(self, ctx):
        pass

    # Exit a parse tree produced by blendedParser#attributeVariable.
    def exitAttributeVariable(self, ctx):
        pass


    # Enter a parse tree produced by blendedParser#stringVar.
    def enterStringVar(self, ctx):
        pass

    # Exit a parse tree produced by blendedParser#stringVar.
    def exitStringVar(self, ctx):
        pass


    # Enter a parse tree produced by blendedParser#identifier.
    def enterIdentifier(self, ctx):
        pass

    # Exit a parse tree produced by blendedParser#identifier.
    def exitIdentifier(self, ctx):
        pass


    # Enter a parse tree produced by blendedParser#text.
    def enterText(self, ctx):
        pass

    # Exit a parse tree produced by blendedParser#text.
    def exitText(self, ctx):
        pass






import json
import sys
import functools
import collections

from jinja2._compat import imap

PY2 = sys.version_info[0] == 2
PY3 = sys.version_info[0] == 3
PY34 = sys.version_info[0:2] >= (3, 4)

class OrderedSet(collections.MutableSet):
    """
    Order set class to created a Ordered set list.
    """

    def __init__(self, iterable=None):
        """
        initialization of OrderSet
        """
        self.end = end = []
        end += [None, end, end]         # sentinel node for doubly linked list
        self.map = {}                   # key --> [key, prev, next]
        if iterable is not None:
            self |= iterable

    def __len__(self):
        """
        magic method for length of set
        """
        return len(self.map)

    def __contains__(self, key):
        """
        return Key from the map
        """
        return key in self.map

    def add(self, key):
        """
        Add method of OrderedSet
        """
        if key not in self.map:
            end = self.end
            curr = end[1]
            curr[2] = end[1] = self.map[key] = [key, curr, end]

    def discard(self, key):
        """
        Discard Method
        """
        if key in self.map:
            key, prev, next = self.map.pop(key)
            prev[2] = next
            next[1] = prev

    def __iter__(self):
        """
        Magic Method Iter to iterate over Set
        """
        end = self.end
        curr = end[2]
        while curr is not end:
            yield curr[0]
            curr = curr[2]

    def __reversed__(self):
        """
        magic method reversed
        """
        end = self.end
        curr = end[1]
        while curr is not end:
            yield curr[0]
            curr = curr[1]

    def pop(self, last=True):
        """
        Pop method for OrderSet
        """
        if not self:
            raise KeyError('set is empty')
        key = self.end[1][0] if last else self.end[2][0]
        self.discard(key)
        return key

    def __repr__(self):
        """
        repr method for OrderedSet
        """
        if not self:
            return '%s()' % (self.__class__.__name__,)
        return '%s(%r)' % (self.__class__.__name__, list(self))

    def __eq__(self, other):
        """
        """
        if isinstance(other, OrderedSet):
            return len(self) == len(other) and list(self) == list(other)
        return set(self) == set(other)

old_set = set
set = OrderedSet

def intersection(list1, list2):
    """
    Returns intersections of two list.
    """
    
    L1 = list1
    L2 = list2
    L = list()
    for v1 in L1:
        for v2 in L2:
            if v1 == v2 and v1 not in L:
                L.append(v1)
    return L
   
def union(list1, list2):
    """
    Returns union of two lists.
    """
    
    mapped_list1, mapped_list2 = mapped_list(list1, list2)
    set_union = set(mapped_list1) | set(mapped_list2)
    result = result_set(set_union)
    return result

def symmetric_difference(list1, list2):
    """
    Returns symmetric difference of two lists.
    """
    mapped_list1, mapped_list2 = mapped_list(list1, list2)
    set_difference = set(mapped_list1) ^ set(mapped_list2)
    result = result_set(set_difference)
    return result

def complement(list1, list2):
    """
    Returns Complement of two lists.
    """
    mapped_list1, mapped_list2 = mapped_list(list1, list2)
    set_list1, set_list2 = set(mapped_list1), set(mapped_list2)
    set_complement = set_list1 - set_list2
    result = result_set(set_complement)
    return result

def cartisan_product(list1, list2):
    """
    Returns cartisan product of two lists.
    """
    result = [[x, y]  for x in list1 for y in list2]
    if PY2:
        cartisan_product_list = _byteify(result)
    elif PY3 or PY34:
        cartisan_product_list = result
    cartisan_product_result_list = list(imap(functools.partial(
        json.dumps, sort_keys=True), cartisan_product_list))
    cartisan_product_set = set(cartisan_product_result_list)
    if PY3 or PY34:
        cartisan_product_result = list(imap(functools.partial(
            json.loads, encoding="utf-8"), cartisan_product_set))
    elif PY2:
        cartisan_product_result = list(imap(json_loads_byteified, cartisan_product_set))
    return cartisan_product_result

def mapped_list(list1, list2):
    """
    Returns list mapped in json string.
    """
    mapped_list1 = list(imap(functools.partial(json.dumps, sort_keys=True), list1))
    mapped_list2 = list(imap(functools.partial(json.dumps, sort_keys=True), list2))
    return mapped_list1, mapped_list2

def result_set(set_obj):
    """
    Take set object as argument and return a list object for the given set
    """
    if PY3 or PY34:
        result = list(imap(functools.partial(json.loads, encoding="utf-8"), set_obj))
    elif PY2:
        result = list(imap(json_loads_byteified, set_obj))
    return result

def json_loads_byteified(json_text):
    """
    returns byteified of the give data in json representation of data
    """
    return _byteify(
        json.loads(json_text, object_hook=_byteify),
        ignore_dicts=True)

def _byteify(data, ignore_dicts=False):
    """
    byteify the unicode in python2.
    """
    # if this is a unicode string, return its string representation
    if isinstance(data, unicode):
        return data.encode('utf-8')
    # if this is a list of values, return list of byteified values
    if isinstance(data, list):
        return [_byteify(item, ignore_dicts=False) for item in data]
    # if this is a dictionary, return dictionary of byteified keys and values
    # but only if we haven't already byteified it
    if isinstance(data, dict) and not ignore_dicts:
        return {
            _byteify(key, ignore_dicts=False): _byteify(value, ignore_dicts=False)
            for key, value in data.iteritems()
        }
    # if it's anything else, return it in its original form
    return data



try:
    from django.conf.urls.defaults import *
except ImportError:
    from django.conf.urls import *

from blendedUxLang.blended.djangolint.views import validate

urlpatterns = [
   url(r'^validate/$', validate),    
]



import os
import sys
import warnings
import collections
import datetime
import json

from copy import copy
from numbers import Number

from django.template import Template
from django.template import Library, Context, RequestContext, Variable, TemplateSyntaxError
from django.template.loader import get_template
from django.template.loader_tags import ExtendsNode, BlockNode
from django.utils.safestring import mark_safe, EscapeText

try:
    from django.template.defaultfilters import escape_filter
except ImportError:
    from django.template.defaultfilters import escape as escape_filter
try:
    from django.utils.encoding import force_text 
except ImportError:
    # This doesn't really do anything, but it probably should. Here for old Django compatibility.
    def force_text(s, *args, **kwargs):
        return str(s)
try:
    from django.utils.deprecation import RemovedInDjango110Warning as DjangoDeprecationWarning
except ImportError:
    DjangoDeprecationWarning = DeprecationWarning
try:
    from django.utils.six import string_types
except ImportError:
    string_types = basestring
try:
    from django.template.base import Template as BaseTemplate
    template_types = (Template, BaseTemplate)
except ImportError:
    template_types = Template
try:
    from django.template.base import render_value_in_context
except ImportError:
    from django.template.base import _render_value_in_context as render_value_in_context

from customtags import core, arguments, utils
from customtags._compat import text_type
from customtags.trimfloat import trimfloat
from blendedUxLang.blended.djangotags.loader import template_from_string
from blendedUxLang.blended.functions import builtins as builtin_funcs

register = Library()

datetime_isoformat = "%Y-%m-%dT%H:%M:%SZ"
numbers_default_string_format = "%g"
PYTHON_MAJOR_VER = sys.version_info[0]


def map_func(function, *iterables):
    if PYTHON_MAJOR_VER<3:
        from itertools import imap
        return imap(function, *iterables)
    elif PYTHON_MAJOR_VER>=3: #Assuming future python versions to be more compatible with py3
        return map(function, *iterables)
    else:
        raise ("Python version incompatible")       

class Parent(core.Tag):
    parent_var = Variable('block.super')

    def render_tag(self, context):
        return self.parent_var.resolve(context)


class Print(core.Tag):
    options = core.Options(arguments.Argument("expression"))

    def render_tag(self, context, expression):
        expression = '' if expression is None or (isinstance(expression, EscapeText) and expression == 'None') else expression
        return render_value_in_context(expression, context)


class Do(core.Tag):
    options = core.Options(arguments.Argument("expression"))

    def render_tag(self, context, expression):
         return ""


class Set(core.Tag):
    options = core.Options(
        arguments.Argument("variable", resolve=False),
        arguments.Constant("="),
        arguments.Expression("value")
    )

    def render_tag(self, context, variable, value):
        context[variable] = value
        return ""


class Verbatim(core.Tag):
    options = core.Options(
        arguments.Literal('literal'),
        arguments.EndTag()
    )

    def render_tag(self, context, literal):
        return literal


class Autoescape(core.Tag):
    options = core.Options(
        arguments.Argument("autoescape_on"),
        arguments.NodeList("nodelist"),
        arguments.EndTag()
    )

    def render_tag(self, context, autoescape_on, nodelist):
        old_setting = context.autoescape
        context.autoescape = autoescape_on
        output = nodelist.render(context)
        context.autoescape = old_setting
        if autoescape_on:
            return mark_safe(output)
        else:
            return output


def get_macro_context(context):
    rc = context.render_context
    macro_context = rc['macros'] = rc.get('macros', Context())
    return macro_context


def set_macro_context(context, macro_context):
    rc = context.render_context
    rc['macros'] = macro_context


def get_new_context(context, initial_data=None):
    if not initial_data:
        initial_data = {}

    ## the request context will allow us to populate the context with all
    ## global/default data, but that is only available if we already have a 
    ## RequestContext via context processors 

    if isinstance(context, RequestContext):
        #new_context = RequestContext(context.request, initial_data, autoescape=context.autoescape)
        from copy import copy, deepcopy
        """
        Instantiating RequestContext constructure is causing
        all the context_processors to execute everytime this is being called.
        # deepcopy throwing TypeError("cannot serialize '_io.BufferedReader' object")
        """
        new_context = copy(context)
        new_context.update(builtin_funcs())
    ## If we don't have a RequestContext, then we need to invoke the builtins
    ## context processor directly.  User functions will not be available
    ## inside the macro body unless added originally to the macro context.

    else:
        new_context = Context(autoescape=context.autoescape)
        new_context.update(builtin_funcs())
        new_context.update(initial_data)

    return new_context


class Macro(core.Tag):
    options = core.Options(
        arguments.Argument("macroname", resolve=False),
        arguments.Constant("("),
        arguments.MultiValueArgument("arg_names", resolve=False,
                                     required=False, commas=True),
        arguments.Constant(")"),
        arguments.NodeList("nodelist"),
        arguments.EndTag()
    )

    def render_tag(self, context, macroname, arg_names, nodelist):
        macro_context = get_macro_context(context)
        
        def macro(*args):
            #if len(arg_names) != len(args):
            #    raise TypeError("Macro '%s' expects %s arguments, %s provided." % 
            #                    (macroname, len(arg_names), len(args)))

            #macro_context = get_macro_context(context)
            macro_context.push()

            ## get a fully scoped context with the macros as initial data
            inner_context = get_new_context(context, copy(macro_context))
            set_macro_context(inner_context, macro_context)
            i = 0
            for arg_name in arg_names:
                try:
                  inner_context[arg_name] = args[i]
                except IndexError:
                  break
                i += 1
            rendered = nodelist.render(inner_context)
            macro_context.pop()
            return rendered
        macro.is_macro = True
        macro.name = macroname

        #macrocontext = get_macro_context(context)
        macro_context[macroname] = context[macroname] = macro
        return ""


def import_macros(context, template):

    # Does this quack like a Template?
    if not callable(getattr(template, 'render', None)):
        # If not, as in include, we'll try get_template
        template = get_template(template)

    local_context = get_new_context(context)
    ## in future versions of Django, base templates may be wrapped by engine templates
    getattr(template, 'template', template).render(local_context)
    macro_dict = {}
    for sub in local_context:
        for key, value in sub.items():
            if getattr(value, 'is_macro', False) and key not in macro_dict:
                macro_dict[key] = value
    return macro_dict


class Import(core.Tag):
    options = core.Options(
        arguments.Argument("template"),
        arguments.Constant("as"),
        arguments.Argument("asname", resolve=False)
    )

    def render_tag(self, context, template, asname):
        context[asname] = import_macros(context, template)
        return ""


class From(core.Tag):
    options = core.Options(
        arguments.Argument("template"),
        arguments.Constant("import"),
        arguments.Argument("name", resolve=False),
        arguments.Optional(
            arguments.Constant("as"),
            arguments.Argument("asname", resolve=False)
        ),
        arguments.Repetition(
            "names",
            arguments.Constant(","),
            arguments.Argument("name", resolve=False),
            arguments.Optional(
                arguments.Constant("as"),
                arguments.Argument("asname", resolve=False)
            ),
        )
    )

    def render_tag(self, context, template, name, asname=None, names=None):
        macros_dict = import_macros(context, template)
        if name not in macros_dict:
            raise ImportError("Macro name '%s' not found in template." % name)
        if asname:
            context[asname] = macros_dict[name]
        else:
            context[name] = macros_dict[name]
        for nameobj in names:
            name = nameobj['name']
            if name not in macros_dict:
                raise ImportError("Macro name '%s' not found in template." % name)
            if 'asname' in nameobj:
                context[nameobj['asname']] = macros_dict[nameobj['name']]
            else:
                context[nameobj['name']] = macros_dict[nameobj['name']]
        return ""


class Include(core.Tag):
    options = core.Options(
        arguments.Argument("template"),
        arguments.Optional(
            arguments.Constant("ignore"),
            arguments.Constant("missing"),
        ),
        arguments.OneOf(
            arguments.MultiArgument(
                arguments.Constant("with"),
                arguments.OneOf(
                    arguments.MultiValueKeywordArgument("extra_context", required=True),
                    arguments.Argument("extra_context"),
                ),
                arguments.Flag('isolation', true_values=['only'], default=False, 
                               case_sensitive=True)
            ),
            arguments.MultiArgument(
                arguments.Flag('isolation', true_values=['only'], case_sensitive=True),
                arguments.Constant("with"),
                arguments.OneOf(
                    arguments.MultiValueKeywordArgument("extra_context", required=True),
                    arguments.Argument("extra_context"),
                )
            ),
            arguments.Flag('isolation', true_values=['only'], default=False, 
                           case_sensitive=True),
        ),
    )

    def render_tag(self, context, template, extra_context=None, isolation=False, ignore=False):
        try:
            extra_context = extra_context if extra_context else {}

            if not isinstance(extra_context, collections.Mapping):
                raise TemplateSyntaxError(
                    "Extra context passed to include tag must act like a dict.")

            # Does this quack like a Template?
            if not callable(getattr(template, 'render', None)):
                # If not, we'll try get_template
                template = context.template.engine.get_template(template)
            old_ae_setting = context.autoescape
            if old_ae_setting:
                context.autoescape = not old_ae_setting
            if isolation:
                rendered = template.render(get_new_context(context, extra_context))
                context.autoescape = old_ae_setting
                return rendered

            ## this should be implemented as 'with context.push(**extra_context)'
            ## but we can't do that because we are supporting old Django
            context.update(extra_context)
            rendered = template.render(context)
            context.pop()
            context.autoescape = old_ae_setting
            return rendered

        except Exception as e:
            if hasattr(context, 'template'):
                if context.template.engine.debug:
                    raise
                return ''
            else:
                from django.conf import settings
                if settings.TEMPLATE_DEBUG:
                    raise
                return ''


class If(core.Tag):
    options = core.Options(
        arguments.Argument('conditional'),
        arguments.NodeList('nodelist'),
        arguments.Repetition(
            'elifs',
            arguments.BlockTag(
                'elif',
                arguments.Argument('conditional'),
                arguments.NodeList('nodelist', endtags=['elif','else','endif']),
            ),
        ),
        arguments.Optional(
            arguments.BlockTag('else'),
            arguments.NodeList('elsenodelist'),
        ),
        arguments.EndTag()
    )

    def render_tag(self, context, conditional, nodelist, elifs, elsenodelist=None):
        if conditional:
            return nodelist.render(context)
        for block in elifs:
            if block['conditional']:
                return block['nodelist'].render(context)
        if elsenodelist:
            return elsenodelist.render(context)
        return ""

class Ifblock(core.Tag):
    name = 'ifblock'
    options = core.Options(
        arguments.MultiValueArgument('block_names', resolve=False),
        arguments.NodeList('nodelist'),
        arguments.Optional(
            arguments.BlockTag('else'),
            arguments.NodeList('elsenodelist'),
        ),
        arguments.EndTag()        
    )
    
    def render_tag(self, context, block_names, nodelist, elsenodelist=None):
        nodes_list = context.template.nodelist.get_nodes_by_type(BlockNode)
        result = []
        for node in nodes_list:
            if node.name in block_names:
                exp_value = node.render(context).strip()
            else:
                exp_value = ''        
            result.append(exp_value)
        conditional = ''
        for val in result:
            conditional = (conditional or val)
        if conditional:
            return nodelist.render(context)
        if elsenodelist:
            return elsenodelist.render(context)        
        return '' 

class For(core.Tag):
    options = core.Options(
        arguments.MultiValueArgument('loopvars', resolve=False, commas=True),
        arguments.Constant('in'),
        arguments.Argument('values'),
        arguments.Flag('is_reversed', true_values=['reversed'], default=False),
        arguments.NodeList('pre_empty'),
        arguments.Optional(
            arguments.OneOf(
                arguments.BlockTag('empty'),
                arguments.BlockTag('else'),
            ),
            arguments.NodeList('post_empty'),
        ),
        arguments.EndTag()
    )

    def render_tag(self, context, loopvars, values, is_reversed, pre_empty, post_empty=None):
        if 'forloop' in context:
            parentloop = context['forloop']
        else:
            parentloop = {}
        #with context.push():

        context.push()
        if values is None:
            values = []
        if not hasattr(values, '__len__'):
            values = list(values)
        len_values = len(values)
        if len_values < 1:
            retval = post_empty.render(context)
        else:
            nodelist = []
            if is_reversed:
                values = reversed(values)
            num_loopvars = len(loopvars)
            unpack = num_loopvars > 1
            # Create a forloop value in the context.  We'll update counters on each
            # iteration just below.
            loop_dict = context['forloop'] = context['loop'] = {'parentloop': parentloop}
            for i, item in enumerate(values):
                # Shortcuts for current loop iteration number.
                loop_dict['counter0'] = loop_dict['index0'] = i
                loop_dict['counter'] = loop_dict['index'] = i + 1
                # Reverse counter iteration numbers.
                loop_dict['revcounter'] = loop_dict['revindex'] = len_values - i
                loop_dict['revcounter0'] = loop_dict['revindex0'] = len_values - i - 1
                # Boolean values designating first and last times through loop.
                loop_dict['first'] = (i == 0)
                loop_dict['last'] = (i == len_values - 1)

                pop_context = False
                if unpack:
                    # If there are multiple loop variables, unpack the item into
                    # them.

                    # To complete this deprecation, remove from here to the
                    # try/except block as well as the try/except itself,
                    # leaving `unpacked_vars = ...` and the "else" statements.
                    if not isinstance(item, (list, tuple)):
                        len_item = 1
                    else:
                        len_item = len(item)
                    # Check loop variable count before unpacking
                    if num_loopvars != len_item:
                        warnings.warn(
                            "Need {} values to unpack in for loop; got {}. "
                            "This will raise an exception in Django 1.10."
                            .format(num_loopvars, len_item),
                            DjangoDeprecationWarning)
                    try:
                        unpacked_vars = dict(zip(loopvars, item))
                    except TypeError:
                        pass
                    else:
                        pop_context = True
                        context.update(unpacked_vars)
                else:
                    context[loopvars[0]] = item
                # In debug mode provide the source of the node which raised
                # the exception
                try:
                    debug_mode = context.template.engine.debug
                except:
                    from django.conf import settings
                    debug_mode = settings.TEMPLATE_DEBUG

                if debug_mode:
                    for node in pre_empty:
                        try:
                            nodelist.append(node.render(context))
                        except Exception as e:
                            if not hasattr(e, 'django_template_source'):
                                e.django_template_source = node.source
                            raise
                else:
                    for node in pre_empty:
                        nodelist.append(node.render(context))
                if pop_context:
                    # The loop variables were pushed on to the context so pop them
                    # off again. This is necessary because the tag lets the length
                    # of loopvars differ to the length of each set of items and we
                    # don't want to leave any vars from the previous loop on the
                    # context.
                    context.pop()
            retval = mark_safe(''.join(force_text(n) for n in nodelist))

        context.pop()
        return retval


extends_options = core.Options(arguments.Expression())
extends_options.initialize('extends')

def do_extends(parser, token):

    container = utils.Container()
    extends_options.parse(parser, token, container)

    parent_name = container.tag_args[0]
    parent_name.token = token
    nodelist = parser.parse()
    if nodelist.get_nodes_by_type(ExtendsNode):
        raise TemplateSyntaxError(
            "'%s' cannot appear more than once in the same template" % bits[0])
    try:
        return ExtendsNode(nodelist, parent_name)
    except TypeError:
        return ExtendsNode(nodelist, None, parent_name)


def items_filter(dict_val):
    try:
        keys = sorted(dict_val.keys())
    except:
        keys = dict_val.keys() 
        keys.sort()
    return [(key, dict_val[key]) for key in keys]


def number_filter(value, default=0):
    """Convert the value into an integer. If the
    conversion doesn't work it will return ``0``. You can
    override this default using the first parameter. You
    can also override the default base (10) in the second
    parameter, which handles input with prefixes such as
    0b, 0o and 0x for bases 2, 8 and 16 respectively.
    """
    try:
        return float(value)
    except (TypeError, ValueError):
        return default


def template_filter(template):
    if isinstance(template, string_types):
        template_obj = template_from_string(template)
        return template_obj
    elif isinstance(template, template_types):
        return template
    else:
        #raise TypeError("The `template` filter must be applied only to strings or templates.")
        ## or just convert it to a string first:
        template_obj = template_from_string(str(template))
        return template_obj


def render_filter(template, context=None, autoescape=None):
    autoescape = False if autoescape is None else autoescape

    if context is None:
        context = {}
    elif not isinstance(context, (dict, Context)):
        raise TypeError("The `render` filter must be passed a dict or Context as an argument.")
    if not hasattr(template, 'render'):
        raise TypeError("The `render` filter should only be applied to Template instances.")

    new_context = Context(context, autoescape=autoescape)
    new_context.update(builtin_funcs())
    return template.render(new_context)


def string_filter(value, format=None, recursion= False):
    #return text_type(value)
    
    try: #try except block to fix the absence of unicode in py3
        if value and isinstance(value, unicode):
            value = value.encode('utf-8')
    except NameError:
        pass

    if format:
        if isinstance(value, Number) and not isinstance(value, bool):
            return format % value
        if isinstance(value, str):
            return format % trimfloat(value)
        elif isinstance(value, datetime.datetime):
            return datetime.datetime.strftime(value, format)
        elif isinstance(value, (list, tuple)):
            rendered = []
            for item in value:
                rendered.append(string_filter(item))
            return format % tuple(rendered)
        else:
            format % string_filter(value)
    else:
        if value is None:
            return 'null'
        elif isinstance(value, bool):
            return 'true' if value else 'false'
        elif isinstance(value, str):              
            if recursion:
                return "'%s'" % value
            else:
                return "%s" %  value   
        elif isinstance(value, Number):
            return str(trimfloat(value))
        elif isinstance(value, datetime.datetime ):
             return datetime.datetime.strftime(value, datetime_isoformat)
        elif isinstance(value, (list, tuple)):
            rendered = []
            for item in value:
                rendered.append(string_filter(item, recursion= True))
            return "[%s]" % ", ".join(rendered)
        elif isinstance(value, dict):
            rendered = []
            for key in sorted(value.keys()):
                rendered_value = string_filter(value[key], recursion= True)
                rendered.append('\'%s\': %s' % (key, rendered_value))
            return "{%s}" % ", ".join(rendered)
        else:
            raise TypeError("Type %s unsupported by string filter" % value.__class__)


def datetime_filter(obj, formatting=None):
    """this filter accepts string, list or dates and return a datetime object.
    """
    if isinstance(obj, datetime.datetime):
        return obj
    elif isinstance(obj, text_type):
        if formatting:
            return datetime.datetime.strptime(obj, formatting)
        else:
            return datetime.datetime.strptime(obj, datetime_isoformat)
    elif isinstance(obj, list):
        return datetime.datetime(*map_func(int, obj), tzinfo=None)


def is_valid_json(json_string):
    try:
        json.loads(json_string)
        return True
    except json.JSONDecodeError:
        return False

def Object(value): 
    try:
        value = value.replace("'", "\"")
        if is_valid_json(value):
            object = json.loads(value)
            if (isinstance(object, dict) and not isinstance(object, list)):
                return object
    except Exception:
        raise Exception('Provided String is not a valid Json Object')
        

def Array(value):
    try: 
        value = value.replace("'", "\"")
        if is_valid_json(value):
            array = json.loads(value)
            if (isinstance(array, list) and not isinstance(array, dict)):
                return array
    except Exception:
        raise Exception('Provided String is not a valid Json Object')

register.tag(Parent.as_tag())
register.tag(Print.as_tag())
register.tag(Do.as_tag())
register.tag(Set.as_tag())
register.tag(Verbatim.as_tag())
register.tag(Autoescape.as_tag())
register.tag(Macro.as_tag())
register.tag(Import.as_tag())
register.tag(From.as_tag())
register.tag(Include.as_tag())
register.tag(If.as_tag())
register.tag(Ifblock.as_tag())
register.tag(For.as_tag())
register.tag('extends', do_extends)
register.filter('items', items_filter)
register.filter('number', number_filter)
register.filter('e', escape_filter)
register.filter('escape', escape_filter)
register.filter('string', string_filter)
register.filter('template', template_filter)
register.filter('render', render_filter, needs_autoescape=True)
register.filter('datetime', datetime_filter)
register.filter('object', Object)
register.filter('array', Array)



import sys

from antlr4 import *
from antlr4.tree.Trees import Trees

from antlr4.error.ErrorListener import ErrorListener

is_py3k = sys.version_info[0] > 2

if is_py3k:
    from blendedUxLang.blended.lint.blended_grammar.py3.blendedLexer import blendedLexer
    from blendedUxLang.blended.lint.blended_grammar.py3.blendedParser import blendedParser
else:
    from blendedUxLang.blended.lint.blended_grammar.py2.blendedLexer import blendedLexer
    from blendedUxLang.blended.lint.blended_grammar.py2.blendedParser import blendedParser


from blendedUxLang.blended.lint.listeners import BlendedErrorListener
    
def lint(input):
    error_listener = BlendedErrorListener() 

    lexer = blendedLexer(input)
    lexer.removeErrorListeners()
    lexer.addErrorListener(error_listener)

    stream = CommonTokenStream(lexer)

    parser = blendedParser(stream)
    parser.removeErrorListeners()
    parser.addErrorListener(error_listener)

    tree = parser.start()

    return error_listener.errors
#error_listener.errors = []




import re
import math
import datetime

from blendedUxLang.blended.sets import intersection, union,\
     symmetric_difference, complement, cartisan_product
from blendedUxLang.blended.colormap import color_map
from blendedUxLang.blended._compat import string_types

HEX_COLOR_RE = re.compile(r'^#([a-fA-F0-9]{3}|[a-fA-F0-9]{6})$')
HEX_COLOR_RE_WITHOUT_HASH = re.compile(r'^([a-fA-F0-9]{3}|[a-fA-F0-9]{6})$')

def float_range(start, end=None, increment=None):
    """Take float and integer value as input argument and return range.
    work same as python range.
    """
    if end is None and increment is None:
        return range(int(start))
    elif increment is None:
        return range(int(start), int(end))
    else:
        return range(int(start), int(end), int(increment))

def cycle(array, index):
    """
    """
    length = len(array)
    return array[int(index) % length]

def first(list_obj):
    """returns the first element in the list passed in"""
    if len(list_obj) > 0:
        return list_obj[0]
    return None

def last(list_obj):
    """returns the last element in the list passed in"""
    length = len(list_obj)
    if length > 0:
        return list_obj[-1]
    return None

def join(list1, separator=''):
    """
    Concatenates all the string elements in the list.
    If arg2 is passed in, the elements are separated by that string.
    """
    separator = str(separator)
    try:
        str_var = separator.join(str(val) for val in list1)
    except TypeError:
        return "list content non string object"
    return str_var

def concat(list1, list2):
    """
    takes 2 lists and joins them together.
    returns the result
    """
    return list1 + list2

def length(value):
    """returns length of string or number of items in list"""
    if isinstance(value, (list, str)):
        length = len(value)
    elif isinstance(value, (int, float)):
        val = str(value)
        length = len(val)
    if length:
        return length
    return None

def lower(arg):
    """converts string passed in to all lower case and returns result"""
    try:
        arg = str(arg)
    except TypeError:
        return "argument in lower function must be string, int or float"
    if isinstance(arg, str):
        return arg.lower()
    return None

def upper(arg):
    """converts string passed in to all upper case and returns result"""
    try:
        arg = str(arg)
    except TypeError:
        return "argument in upper function must be string, int or float"
    if isinstance(arg, str):
        return arg.upper()
    return None

def title(arg):
    """Converts string passed in to title case and returns result"""
    try:
        arg = str(arg)
    except TypeError:
        return "argument in title function must be string, int or float"
    if isinstance(arg, str):
        return arg.title()
    return None

def blendedround(number, precision=0):
    """rounds a number to the nearest whole number"""
    if (number.__class__.__name__ == 'Markup'):
        number = float(number.unescape().strip("'"))
    try:
        return round(number, precision)
    except TypeError:
        return round(float(str(number)), precision)

def ceil(value):
    """Rounds the value up to the nearest int."""
    if not isinstance(value, float):
        value = float(value)
    if isinstance(value, float):
        return math.ceil(value)
    return None

def floor(value):
    """Rounds down to the next lowest integer."""
    if not isinstance(value, float):
        value = float(value)
    if isinstance(value, float):
        return math.floor(value)
    return None

def mean(values):
    """Return mean of the given values.
    """
    return float(sum(values) / len(values) if len(values) > 0 else None)

def rgbcolor(colorvalue):
    """
    This function takes a hex color and
    returns a list with 3 ints representing the rgb numbers
    """
    if isinstance(colorvalue, (list, tuple)) and len(colorvalue) > 3:
        return [0, 0, 0]
    if isinstance(colorvalue, (list, tuple)) and len(colorvalue) == 3:
        col_r = int(colorvalue[0])
        col_g = int(colorvalue[1])
        col_b = int(colorvalue[2])
        if ((col_r <= 255 and col_r >= 0)
                and (col_g <= 255 and col_g >= 0)
                and (col_b <= 255 and col_b >= 0)):
            return [col_r, col_g, col_b]
        else:
            return [0, 0, 0]
    elif isinstance(colorvalue, string_types):
        if colorvalue.startswith('#'):
            normalisevalue = normalize_hex(colorvalue)
            value = normalisevalue.lstrip('#')
        else:
            hexvalue = name_to_hex(colorvalue)
            value = hexvalue.lstrip('#')
        length_value = len(value)
        try:
            if length_value == 1:
                return_value = int(value, 16)*17
                col = return_value, return_value, return_value
                return list(col)
            if length_value == 3:
                return list(int(value[i:i+1], 16)*17 for i in range(0, 3))
            length_value2 = int(length_value/3)
            return list(int(value[i:i+length_value2], 16) for i in range(0, length_value, length_value2))
        except ValueError:
            return [0, 0, 0]

def hexcolor(colorvalue):
    """
    takes a list of 3 ints and returns string representing Hex value
    """
    if isinstance(colorvalue, (list, tuple)) and len(colorvalue) > 3:
        return "#000000"
    if isinstance(colorvalue, (list, tuple)) and len(colorvalue) == 3:
        col_r = int(colorvalue[0])
        col_g = int(colorvalue[1])
        col_b = int(colorvalue[2])
        if ((col_r <= 255 and col_r >= 0)
                and (col_g <= 255 and col_g >= 0)
                and (col_b <= 255 and col_b >= 0)):
            return '#%02X%02X%02X' % (col_r, col_g, col_b)
        else:
            return "#000000"
    elif isinstance(colorvalue, string_types) and colorvalue.startswith('#'):
        hexvalue = normalize_hex(colorvalue)
        return hexvalue
    elif isinstance(colorvalue, string_types):
        hexvalue = name_to_hex(colorvalue)
        return hexvalue

def normalize_hex(hex_value):
    """
    Normalize a hexadecimal color value to 6 digits, lowercase.
    """
    if hex_value.startswith('#'):
        match = HEX_COLOR_RE.match(hex_value)
    else:
        match = HEX_COLOR_RE_WITHOUT_HASH.match(hex_value)
    if match is None:
        return "#000000"
    hex_digits = match.group(1)
    if len(hex_digits) == 3:
        hex_digits = ''.join(2 * s for s in hex_digits)
    return '#%s' % hex_digits.upper()

def name_to_hex(name):
    """
    Convert a color name to a normalized hexadecimal color value.
    When no color of that name exists in the color_map,
    ``ValueError`` is raised.
    """
    normalized = name.lower()
    hex_value = color_map.get(normalized)
    if hex_value is None:
        return normalize_hex(name)
    hexvalue = hex_value.upper()
    return hexvalue

def series(start, length, increment=None):
    """returns a range from a given start value upto the given length,
    with an optional argument 'increment'.
    """
    arr = []
    if not increment:
        increment = 1
    i = start
    while length > 0:
        arr.append(i)
        i += increment
        length -= 1
    return arr

def now():
    """
    Return a datetime object for the current time of the day
    """
    return datetime.datetime.now()

def builtins(request=None):
    """
    return builtin fucitons
    """
    return {
        'range':float_range,
        'cycle':cycle,
        'first':first,
        'last':last,
        'join':join,
        'concat':concat,
        'length':length,
        'lower':lower,
        'upper':upper,
        'title':title,
        'round':blendedround,
        'ceil':ceil,
        'floor':floor,
        'abs':abs,
        'mean':mean,
        'hexcolor':hexcolor,
        'rgbcolor':rgbcolor,
        'product':cartisan_product,
        'union':union,
        'complement':complement,
        'difference':symmetric_difference,
        'intersection':intersection,
        'series':series,
        'now':now,
    }



import json

from django.http import HttpResponse
from django.shortcuts import render
from django.views.decorators.csrf import csrf_exempt

from antlr4.InputStream import InputStream
from blendedUxLang.blended.lint.lint import lint


@csrf_exempt
def validate(request):
    """
    view to return error messages if passed string is invalid.
    """    
    try:
        message = request.POST['message']
    except KeyError:
        return HttpResponse("A 'message' must be passed in POST request")
    results = lint(InputStream(message))
    if not results:
        return HttpResponse(json.dumps('No Errors'), content_type='application/json')
    else:
        return HttpResponse(json.dumps(results), content_type='application/json')
            



"""
In all versions of Django, models.py loads at system startup.  So, this is the place
to put any monkey-patching that may be required to bootstrap a system.

Blended does a coupld of things that might be considered "monkey patching".  First, it 
appends the blended_tags library to the template tag builtins.  This way, the Blended
versions of the template tags will override the template tags and filters natively provided
by Django, without users having to add {% load blended_tags %} to every template.  

This automatic loading can be turned off by setting the 'override_builtins' option of the TEMPLATES 
setting to False, or by setting the OVERRIDE_BUILTINS setting itself to False (in case of a 
conflict where one of the two settings is set to True, the TEMPLATES option will take precidence).

Second, in order to provide global control over autoescape functionality, it wraps the constructors
of the django.template.context.Context and django.template.context.RequestContext classes with
decorators that apply the AUTOESCAPE_DEFAULT setting at the time of object instantiation.

By default (i.e. if it is unspecified), the AUTOESCAPE_DEFAULT is False, but it can be overridden
in the settings.  If the original default Django functionality is required, set AUTOESCAPE_DEFAULT
to True.
"""

from django.conf import settings
from blendedUxLang.blended.djangotags.templatetags.blended_tags import register
from django.template.context import Context, RequestContext

TEMPLATES = getattr(settings, 'TEMPLATES', None)

if TEMPLATES and ('OPTIONS' in TEMPLATES) and ('override_builtins' in TEMPLATES['OPTIONS']):
    OVERRIDE_BUILTINS = TEMPLATES['OPTIONS']['override_builtins']
elif getattr(settings, 'OVERRIDE_BUILTINS', None):
    OVERRIDE_BUILTINS = settings.OVERRIDE_BUILTINS
else:
    OVERRIDE_BUILTINS = True

try:
    from django.template.base import builtins
    if OVERRIDE_BUILTINS:
        builtins.append(register)
except ImportError:
    from django.template.engine import Engine
    if OVERRIDE_BUILTINS:
        Engine.default_builtins.append('blended.djangotags.templatetags.blended_tags')
    


def wrap_context_init(__init__):
    def __wrapinit__(self, dict_=None, autoescape=None, **kwargs):
        if autoescape is None:
            autoescape = getattr(settings, 'AUTOESCAPE_DEFAULT', False)
        return __init__(self, dict_, autoescape=autoescape, **kwargs) 
    return __wrapinit__


def wrap_request_context_init(__init__):
    def __wrapinit__(self, request, dict_=None, processors=None, **kwargs):
        autoescape_default = getattr(settings, 'AUTOESCAPE_DEFAULT', False)
        autoescape = kwargs.pop('autoescape', autoescape_default)
        try:
            return __init__(self, request, dict_, processors, autoescape=autoescape, **kwargs) 
        except TypeError:
            return __init__(self, request, dict_, processors, **kwargs)
    return __wrapinit__


Context.__init__ = wrap_context_init(Context.__init__)
RequestContext.__init__ = wrap_request_context_init(RequestContext.__init__)





import sys
import re
import datetime
import ast

from collections import Mapping
from numbers import Number
from functools import reduce

import jinja2
import jinja2.ext
import jinja2.sandbox
import json

from jinja2 import nodes
from jinja2.runtime import Undefined, Macro
from jinja2.environment import TemplateExpression
from jinja2._compat import encode_filename, string_types, iteritems,\
     text_type, imap
from jinja2.utils import Markup
from jinja2.defaults import BLOCK_START_STRING,\
    BLOCK_END_STRING, VARIABLE_START_STRING, VARIABLE_END_STRING,\
    COMMENT_START_STRING, COMMENT_END_STRING, LINE_STATEMENT_PREFIX,\
    LINE_COMMENT_PREFIX, TRIM_BLOCKS, NEWLINE_SEQUENCE,\
    KEEP_TRAILING_NEWLINE, LSTRIP_BLOCKS

from blendedUxLang.blended.functions import builtins
from blendedUxLang.blended.trimfloat import trimfloat, trimint

datetime_isoformat = "%Y-%m-%dT%H:%M:%SZ"
numbers_default_string_format = "%g"

class BlendedTemplate(jinja2.environment.Template):
    """Template class for Blended"""
    spontaneous_environments = jinja2.utils.LRUCache(10)

    def __new__(cls, source,
                block_start_string=BLOCK_START_STRING,
                block_end_string=BLOCK_END_STRING,
                variable_start_string=VARIABLE_START_STRING,
                variable_end_string=VARIABLE_END_STRING,
                comment_start_string=COMMENT_START_STRING,
                comment_end_string=COMMENT_END_STRING,
                line_statement_prefix=LINE_STATEMENT_PREFIX,
                line_comment_prefix=LINE_COMMENT_PREFIX,
                trim_blocks=TRIM_BLOCKS,
                lstrip_blocks=LSTRIP_BLOCKS,
                newline_sequence=NEWLINE_SEQUENCE,
                keep_trailing_newline=KEEP_TRAILING_NEWLINE,
                extensions=(),
                optimized=True,
                undefined=Undefined,
                finalize=None,
                autoescape=False,
                trim_floats=True,
                dict_attrs=False):
        # only changed this to use BlendedEnvironment
        env = cls.get_spontaneous_blended_environment(
            block_start_string, block_end_string, variable_start_string,
            variable_end_string, comment_start_string, comment_end_string,
            line_statement_prefix, line_comment_prefix, trim_blocks,
            lstrip_blocks, newline_sequence, keep_trailing_newline,
            frozenset(extensions), optimized, undefined, finalize, autoescape,
            trim_floats, dict_attrs, None, 0, False, None)
        return env.from_string(source, template_class=cls)

    @classmethod
    def get_spontaneous_blended_environment(cls, *args):
        """This is just implemented here to be able to use BlendedEnvironment"""
        try:
            env = cls.spontaneous_environments.get(args)
        except TypeError:
            return BlendedEnvironment(*args)
        if env is not None:
            return env
        cls.spontaneous_environments[args] = env = BlendedEnvironment(*args)
        env.shared = True
        return env

    def new_context(self, vars=None, shared=False, locals=None, args=None):
        """Reimplementation of Template.new_context,
        needed for handling Include node args.
        """
        if args:
            new_locals = dict(locals) if locals else {}
            for key, value in iteritems(args):
                new_locals['l_' + key] = value
        else:
            new_locals = locals

        return jinja2.runtime.new_context(self.environment, self.name, self.blocks,
                                          vars, shared, self.globals, new_locals)

###
### Extensions and Filters
###

class ParentExtension(jinja2.ext.Extension):
    """Extension for Parent tag."""
    tags = set(['parent'])

    def parse(self, parser):
        """
        """
        lineno = next(parser.stream).lineno
        name_node = nodes.Name('super', 'load', lineno=lineno)
        call_node = nodes.Call(name_node, [], [], None, None, lineno=lineno)
        outp_node = nodes.Output([call_node], lineno=lineno)
        return outp_node

class IfblockExtension(jinja2.ext.Extension):
    """Extension for ifblock tag."""
    tags = set(['ifblock'])

    def parse(self, parser):
        """
        Parser method for ifblock tag
        """
        lineno = parser.stream.expect('name:ifblock').lineno

        params = self.get_inline_params(parser.stream)
        node = result = nodes.If(lineno=lineno)
        param_count = len(params)

        if param_count == 1:
            node.test = self.build_test(params[0], lineno)
        else:
            node.test = self.get_or_expr(params, param_count-1, lineno)

        node.body = parser.parse_statements(('name:else', 'name:endifblock'))
        token = next(parser.stream)
        if token.test('name:else'):
            node.else_ = parser.parse_statements(('name:endifblock',), drop_needle=True)
        else:
            node.else_ = []
        return result

    def get_or_expr(self, params, index, lineno):
        if index == 0:
            return self.build_test(params[0], lineno)
        return nodes.Or(self.get_or_expr(params, index-1, lineno), self.build_test(params[index], lineno), lineno=lineno)

    def build_test(self, block_name, lineno):
        """
        """
        name_node = nodes.Name('self', 'load', lineno=lineno)
        attr_node = nodes.Getattr(name_node, block_name, 'load', lineno=lineno)
        call_node = nodes.Call(attr_node, [], [], None, None, lineno=lineno)
        return nodes.Filter(call_node, 'trim', [], [], None, None, lineno=lineno)

    def get_inline_params(self, parser_stream):
        params = []
        while(parser_stream.current.type != 'block_end' ):
            if (parser_stream.current.type == 'name'):
                params.append(parser_stream.current.value)
                next(parser_stream)
            else:
                raise jinja2.exceptions.TemplateSyntaxError("Expected block name.")
        return params

class CommentExtension(jinja2.ext.Extension):
    """Extension for comment tag."""
    tags = set(['comment'])

    def parse(self, parser):
        token = parser.stream.current
        lineno = next(parser.stream).lineno
        if token.type == 'name' and token.value == 'comment':
            parser.parse_statements(['name:endcomment'], drop_needle=True)
            return nodes.Const("")
        else:
            jinja2.exceptions.TemplateSyntaxError("Expected token 'comment'")

def items_filter(dict_val):
    """iterate over the given object and list of tuple in key, value
    representation.
    """
    if not dict_val:
        return []
    try:
        keys = sorted(dict_val.keys())
    except:
        keys = dict_val.keys()
        keys.sort()
    return [(key, dict_val[key]) for key in keys]

def number_filter(value, default=0):
    """Convert the value into an number. basically in float type
    """
    try:
        return float(value)
    except (TypeError, ValueError):
        return default

@jinja2.evalcontextfilter
def render_filter(eval_ctx, template, context=None):
    """renders the context in the template object.
    """
    if context is None:
        context = {}
    elif not isinstance(context, dict):
        raise TypeError("The `render` filter can only be passed a dict as an argument.")
    if hasattr(template, 'render'):
        return template.render(context)
    else:
        raise TypeError("The `render` filter should only be "
                        "applied to objecst that implement `render`.")

def datetime_filter(obj, formatting=None):
    """this filter accepts string, list or dates and return a datetime object.
    """
    if isinstance(obj, datetime.datetime):
        return obj
    elif isinstance(obj, text_type):
        if formatting:
            return datetime.datetime.strptime(obj, formatting)
        else:
            return datetime.datetime.strptime(obj, datetime_isoformat)
    elif isinstance(obj, list):
        return datetime.datetime(*imap(int, obj), tzinfo=None)

class TemplateNull(object):
    """object of class represents 'null' which is replacing None in string_filter
    """
    def __repr__(self):
        return "null"

class TemplateTrue(object):
    """object of class represents 'true' which is replacing True in string_filter
    """
    def __repr__(self):
        return "true"

class TemplateFalse(object):
    """object of class represents 'false' which is replacing False in string_filter
    """
    def __repr__(self):
        return "false"

null = TemplateNull()
true = TemplateTrue()
false = TemplateFalse()


def string_filter(value, format=None, recursion=False):

    try:  # try except block to fix the absence of unicode in py3
        if value and isinstance(value, unicode):
            value = value.encode('utf-8')
    except NameError:
        pass

    if format:
        if isinstance(value, Number) and not isinstance(value, bool):
            return format % value
        if isinstance(value, str):
            return format % trimfloat(value)
        elif isinstance(value, datetime.datetime):
            return datetime.datetime.strftime(value, format)
        elif isinstance(value, (list, tuple)):
            rendered = []
            for item in value:
                rendered.append(string_filter(item))
            return format % tuple(rendered)
        else:
            format % string_filter(value)
    else:
        if value is None:
            return 'null'
        elif isinstance(value, bool):
            return 'true' if value else 'false'
        elif isinstance(value, str):
            if recursion:
                return "'%s'" % value
            else:
                return "%s" % value
        elif isinstance(value, Number):
            return str(trimfloat(value))
        elif isinstance(value, datetime.datetime):
            return datetime.datetime.strftime(value, datetime_isoformat)
        elif isinstance(value, (list, tuple)):
            rendered = []
            for item in value:
                rendered.append(string_filter(item, recursion=True))
            return "[%s]" % ", ".join(rendered)
        elif isinstance(value, dict):
            rendered = []
            for key in sorted(value.keys()):
                rendered_value = string_filter(value[key], recursion=True)
                rendered.append('\'%s\': %s' % (key, rendered_value))
            return "{%s}" % ", ".join(rendered)
        else:
            raise TypeError("Type %s unsupported by string filter" % value.__class__)

def is_valid_json(json_string):
    try:
        json.loads(json_string)
        return True
    except json.JSONDecodeError:
        return False

def Object(value): 
    try:
        value = value.replace("'", "\"")
        if is_valid_json(value):
            object = json.loads(value)
            if (isinstance(object, dict) and not isinstance(object, list)):
                return object
    except Exception:
        raise Exception('Provided String is not a valid Json Object')
        

def Array(value):
    try: 
        value = value.replace("'", "\"")
        if is_valid_json(value):
            array = json.loads(value)
            if (isinstance(array, list) and not isinstance(array, dict)):
                return array
    except Exception:
        raise Exception('Provided String is not a valid Json Object')



filters = {
    "render": render_filter,
    "items": items_filter,
    "number": number_filter,
    "string": string_filter,
    "datetime": datetime_filter,
    "object": Object,
    "array": Array
}
from jinja2.exceptions import UndefinedError
class BlendedEnvironment(jinja2.sandbox.SandboxedEnvironment):
    """Environment class for blended.
    """
    template_class = BlendedTemplate
    EXTENSIONS = [CommentExtension, ParentExtension, IfblockExtension,
                  jinja2.ext.do, jinja2.ext.with_, jinja2.ext.autoescape]

    def __init__(self,
                 block_start_string=BLOCK_START_STRING,
                 block_end_string=BLOCK_END_STRING,
                 variable_start_string=VARIABLE_START_STRING,
                 variable_end_string=VARIABLE_END_STRING,
                 comment_start_string=COMMENT_START_STRING,
                 comment_end_string=COMMENT_END_STRING,
                 line_statement_prefix=LINE_STATEMENT_PREFIX,
                 line_comment_prefix=LINE_COMMENT_PREFIX,
                 trim_blocks=TRIM_BLOCKS,
                 lstrip_blocks=LSTRIP_BLOCKS,
                 newline_sequence=NEWLINE_SEQUENCE,
                 keep_trailing_newline=True,
                 extensions=(),
                 optimized=True,
                 undefined=Undefined,
                 finalize=None,
                 autoescape=True,
                 loader=None,
                 cache_size=400,
                 auto_reload=True,
                 bytecode_cache=None,
                 trim_floats=True,
                 sandboxed_access=True,
                 dict_attrs=False):
        """
        The new extensions and functions are added to the environment here
        """

        self.trim_floats = trim_floats
        self.dict_attrs = dict_attrs
        self.sandboxed_access = sandboxed_access
        extensions = list(extensions)
        extensions.extend(self.EXTENSIONS)
        super(BlendedEnvironment, self).__init__(block_start_string,
                                                 block_end_string,
                                                 variable_start_string,
                                                 variable_end_string,
                                                 comment_start_string,
                                                 comment_end_string,
                                                 line_statement_prefix,
                                                 line_comment_prefix,
                                                 trim_blocks,
                                                 lstrip_blocks,
                                                 newline_sequence,
                                                 keep_trailing_newline,
                                                 extensions,
                                                 optimized,
                                                 undefined,
                                                 finalize,
                                                 autoescape,
                                                 loader,
                                                 cache_size,
                                                 auto_reload,
                                                 bytecode_cache)

        self.globals.update(builtins())
        self.globals.update({"null": None})

        def template_filter(template):
            """filter to create template from the string"""
            if isinstance(template, jinja2.environment.Template):
                return template
            elif isinstance(template, string_types):
                return self.from_string(template)
            else:
                raise TypeError("The `template` filter must only operate on "
                                "strings or template instances.")

        self.filters['template'] = template_filter
        self.filters.update(filters)

    def __substitute(self, val):
        """Substitution of verbaim to raw performs here
        """
        if re.match(r'''{%\s*verbatim\s*%}(.*?){%\s*endverbatim\s*%}''',
                    val.group(0), re.S):
            return "{%% raw %%}%s{%% endraw %%}" % val.group(1)
        elif re.match(r'''{%\s*verbatim\s*%}''', val.group(0)):
            return "{% raw %}"

    def __add_newline(self, val):
        """New line after endverbatim tag checks here
        If it newline followed by endverbatim tag this regex
        add an additional newline after endverbatim so single
        newline remove when trim_block is true
        """
        content = val.group(1)
        if re.match('\n.*', content, re.S):
            return "{%% endverbatim %%}\n%s" % content
        else:
            return "{%% endverbatim %%}%s" % content

    def preprocess(self, source, name=None, filename=None):
        """Preprocesses the source with all extensions.  This is automatically
        called for all parsing and compiling methods but *not* for :meth:`lex`
        because there you usually only want the actual source tokenized.
        Regex for conversion of verbatim to raw tag.
        """

        if self.trim_blocks:
            source = re.sub(r'''{%\s*endverbatim\s*%}(\n.*?)''', self.__add_newline, source, re.S)
        ptrn = r'''(?s)(?:{%\s*verbatim\s*%}(.*?))?{%\s*endverbatim\s*%}|{%\s*verbatim\s*%}'''
        source = re.sub(ptrn, self.__substitute, source, re.S)

        return reduce(lambda s, e: e.preprocess(s, name, filename),
                      self.iter_extensions(), text_type(source))

    def _parse(self, source, name, filename):
        """overridden to change Parser to BlendedParser"""
        return BlendedParser(self, source, name, encode_filename(filename)).parse()

    def _generate(self, source, name, filename, defer_init=False):
        """
        Should be same algorithm as superclass, but with BlendedCodeGenerator instead.
        This is the place to look at the compiled code.
        """
        if not isinstance(source, nodes.Template):
            raise TypeError('Can\'t compile non template nodes')

        generator = BlendedCodeGenerator(self, name, filename, defer_init=defer_init)
        generator.visit(source)
        result = generator.stream.getvalue()

        return result

    def compile_expression(self, source, undefined_to_none=True):
        """Should be same algorithm as superclass, but with BlendedParser instead"""
        ## Is this ever called ?
        parser = BlendedParser(self, source, state='variable')
        exc_info = None
        try:
            expr = parser.parse_expression()
            if not parser.stream.eos:
                raise jinja2.exceptions.TemplateSyntaxError('chunk after expression',
                                                            parser.stream.current.lineno,
                                                            None, None)
            expr.set_environment(self)
        except jinja2.exceptions.TemplateSyntaxError:
            exc_info = sys.exc_info()
        if exc_info is not None:
            self.handle_exception(exc_info, source_hint=source)
        body = [nodes.Assign(nodes.Name('result', 'store'), expr, lineno=1)]
        template = self.from_string(nodes.Template(body, lineno=1))
        return TemplateExpression(template, undefined_to_none)

    def _trimfloat(self, value):
        """
        trimfloat method to remove fractional part from float value.
        """
        if value==None:
            return trimint(0)
        if self.trim_floats and isinstance(value, Number) and not (isinstance(value, bool) or isinstance(value, complex)):
            if isinstance(value, int):
                return trimint(value)
            else:
                return trimfloat(value)
        try:
            if float(value):
                return trimfloat(value)
        except Exception:
            pass
          
        return value
        

    def getattr(self, obj, attribute):
        """
        """
        if not self.dict_attrs and isinstance(obj, Mapping):
            try:
                return self._trimfloat(obj[attribute])
            except (TypeError, LookupError):
                return self.undefined(obj=obj, name=attribute)
        if self.sandboxed_access:
            return self._trimfloat(super(BlendedEnvironment, self).getattr(obj, attribute))
        else:
            CurrentEnv = self.__class__
            while object not in CurrentEnv.__bases__:
                CurrentEnv = CurrentEnv.__bases__[0]
            return self._trimfloat(CurrentEnv.getattr(self, obj, attribute))

    def getitem(self, obj, argument):
        """
        """
        if not self.dict_attrs and isinstance(obj, Mapping):
            try:
                return self._trimfloat(obj[argument])
            except (TypeError, LookupError):
                return self.undefined(obj=obj, name=argument)
        return self._trimfloat(super(BlendedEnvironment, self).getitem(obj, argument))

    def call(__self, __context, __obj, *args, **kwargs):
        """
        """
        result = super(BlendedEnvironment, __self).call(__context, __obj, *args, **kwargs)
        if isinstance(__obj, Macro) and not isinstance(result, Markup):
            result = Markup(result)
        return __self._trimfloat(result)
        

    def call_filter(self, name, value, args=None, kwargs=None, context=None, eval_ctx=None):
        """
        """
        ## Does this ever get called?
        value = super(BlendedEnvironment, self).call_filter(name, value, args, kwargs,
                                                            context, eval_ctx)
        return self._trimfloat(value)


class BlendedImmutableEnvironment(BlendedEnvironment):
    '''

    '''
    def __init__(self,
                 block_start_string=BLOCK_START_STRING,
                 block_end_string=BLOCK_END_STRING,
                 variable_start_string=VARIABLE_START_STRING,
                 variable_end_string=VARIABLE_END_STRING,
                 comment_start_string=COMMENT_START_STRING,
                 comment_end_string=COMMENT_END_STRING,
                 line_statement_prefix=LINE_STATEMENT_PREFIX,
                 line_comment_prefix=LINE_COMMENT_PREFIX,
                 trim_blocks=TRIM_BLOCKS,
                 lstrip_blocks=LSTRIP_BLOCKS,
                 newline_sequence=NEWLINE_SEQUENCE,
                 keep_trailing_newline=True,
                 extensions=(),
                 optimized=True,
                 undefined=Undefined,
                 finalize=None,
                 autoescape=False,
                 loader=None,
                 cache_size=400,
                 auto_reload=True,
                 bytecode_cache=None,
                 trim_floats=True,
                 sandboxed_access=True,
                 dict_attrs=True):
                 
        super(BlendedImmutableEnvironment, self).__init__(block_start_string,
                 block_end_string, variable_start_string, variable_end_string,
                 comment_start_string, comment_end_string, line_statement_prefix,
                 line_comment_prefix, trim_blocks, lstrip_blocks, newline_sequence,
                 keep_trailing_newline, extensions, optimized, undefined,
                 finalize, autoescape, loader, cache_size, auto_reload,
                 bytecode_cache,trim_floats,sandboxed_access,dict_attrs)
    
    def is_safe_attribute(self, obj, attr, value):
        if not super(BlendedEnvironment, self).is_safe_attribute(obj, attr, value):
            return False
        return not jinja2.sandbox.modifies_known_mutable(obj, attr)


class BlendedParser(jinja2.parser.Parser):
    """
    This parser subclass overrides the parsing of the include tag, in order to provide
    a local context feature.  It also overrides parse_primary to ensure that numbers are
    printed without trailing zeros.
    """

    def _wrap_as_const(self, as_const):
        """
        """
        def as_const_wrapper(eval_ctx=None):
            """
            """
            value = as_const(eval_ctx)
            if isinstance(value, Number):
                return self.environment._trimfloat(value)
            return value
        return as_const_wrapper

    def _wrap_parse(parse_method):
        """
        """
        def parse_wrapper(self, *args, **kwargs):
            """
            """
            node = parse_method(self, *args, **kwargs)
            node.as_const = self._wrap_as_const(node.as_const)
            return node
        return parse_wrapper

    parse_primary = _wrap_parse(jinja2.parser.Parser.parse_primary)
    parse_filter = _wrap_parse(jinja2.parser.Parser.parse_filter)

    def parse_include(self):
        """
        Reimplementation of Parser.parse_include, necessary for new Include functionality
        """
        lineno = lineno = next(self.stream).lineno
        node = nodes.Include(lineno=lineno)
        node.fields = nodes.Include.fields + ('with_expression',) ## compiler needs this

        node.template = self.parse_expression()
        if self.stream.current.test('name:ignore') and\
           self.stream.look().test('name:missing'):
            node.ignore_missing = True
            self.stream.skip(2)
        else:
            node.ignore_missing = False

        node.with_expression = None
        if self.stream.current.test_any('name:with', 'name:without') and\
           self.stream.look().test('name:context'):
            node.with_context = next(self.stream).value == 'with'
            self.stream.skip()
        elif self.stream.current.test('name:with'):
            self.stream.skip()
            node.with_expression = self.parse_expression()
            if self.stream.current.test('name:only'):
                node.with_context = True
                self.stream.skip()
            else:
                node.with_context = True

        elif self.stream.current.test('name:only'):
            node.with_context = False
            self.stream.skip()
        else:
            node.with_context = True
        return node


class BlendedCodeGenerator(jinja2.compiler.CodeGenerator):
    """
    This subclass does a couple of things: it helps standardize all numbers in Jinja
    as floats, but with the feature that trailing zeroes are not printed; it also adds
    functionality to the include tag.

    What is added to include is the ability to pass specific arguments into the tag.

    .. sourcecode:: jinja

        {% include "included_file.html" with { "arg1": strval, "arg2": numval } %}
    """

    def pull_locals(self, frame):
        """
        """
        for name in frame.identifiers.undeclared:
            self.writeline('l_%s = environment._trimfloat(context.resolve(%r))' % (name, name))

    def visit_Const(self, node, frame):
        """
        """
        val = node.value
        if isinstance(val, Number):
            self.write('environment._trimfloat(' + str(val) + ')')
        else:
            self.write(repr(val))

    def visit_Filter(self, node, frame):
        """
        """
        self.write('environment._trimfloat(')
        super(BlendedCodeGenerator, self).visit_Filter(node, frame)
        self.write(')')

    def visit_Include(self, node, frame):
        """
        Handles includes. Overrides base functionality to add new Include node functionality
        """
        if node.with_context:
            self.unoptimize_scope(frame)
        if node.ignore_missing:
            self.writeline('try:')
            self.indent()

        func_name = 'get_or_select_template'
        if isinstance(node.template, nodes.Const):
            if isinstance(node.template.value, string_types):
                func_name = 'get_template'
            elif isinstance(node.template.value, (tuple, list)):
                func_name = 'select_template'
        elif isinstance(node.template, (nodes.Tuple, nodes.List)):
            func_name = 'select_template'

        self.writeline('template = environment.%s(' % func_name, node)
        self.visit(node.template, frame)
        self.write(', %r)' % self.name)
        if node.ignore_missing:
            self.outdent()
            self.writeline('except TemplateNotFound:')
            self.indent()
            self.writeline('pass')
            self.outdent()
            self.writeline('else:')
            self.indent()

        if node.with_expression:
            self.writeline('args = dict(')
            self.visit(node.with_expression, frame)
            self.write(')')

            if node.with_context:
                self.writeline('vars = dict(context.parent)')
                self.writeline('ctxt = template.new_context(vars, True, locals(), args)')
            else:
                self.writeline('ctxt = template.new_context(args, True)')
            self.writeline('for event in template.root_render_func(ctxt):')
        else:
            if node.with_context:
                self.writeline('ctxt = template.new_context(context.parent, True, locals())')
                self.writeline('for event in template.root_render_func(ctxt):')
            else:
                self.writeline('for event in template.module._body_stream:')

        self.indent()
        self.simple_write('event', frame)
        self.outdent()

        if node.ignore_missing:
            self.outdent()



import sys
import os
import click

from antlr4 import FileStream

from blendedUxLang.blended.lint.lint import lint

@click.command()
@click.argument('filename', nargs=-1)
def blended_lint(filename):
    
    working_directory = os.getcwd()
    errors = {}
    filename = list(filename)

    for file in filename:
        file_path = '%s/%s' % (working_directory, file)
        if os.path.isfile(file_path):
            input = FileStream(file_path)
            errors[file] = lint(input)

    if len(errors) == 0:
        if len(filename) > 1:
            click.echo("[ %s ] are not valid files" % ', '.join(filename)[:-1])       
        else:
            click.echo("%s is not a valid file" % filename[0])
    else:
         print_errors(errors)
    


def print_errors(errors):
    for file, errors in errors.items():
        click.echo("---------------------------------")
        click.echo(file)
        if not errors:
            click.echo("No errors! :-)")
        else:
            click.echo("Uh oh!  We have some errors:")
            for error in errors:
                click.echo(error) 
    click.echo("---------------------------------")



if __name__ == '__main__':
    blended_lint()



import math

class trimfloat(float):
    
    def __str__(self):
        """
        Format a float so it prints like PHP and JavaScript, with no trailing zeroes.
        """
        return ('%f' % self).rstrip('0').rstrip('.')
        #('%f' % self).rstrip('0').rstrip('.') 

    def __repr__(self):
        """
        Format a float so it prints like PHP and JavaScript, with no trailing zeroes.
        """
        return ('%f' % self).rstrip('0').rstrip('.')

    def __pos__(self):
        """
        Implements behavior for unary positive (e.g. +some_object)
        """
        
        return self.__class__(float.__pos__(self))

    def __neg__(self):
        """
        Implements behavior for negation (e.g. -some_object)
        """

        return self.__class__(float.__neg__(self))

    def __abs__(self):
        """
        Implements behavior for the built in abs() function.
        """

        return self.__class__(float.__abs__(self))

    def __round__(self, ndigits=0):
        """
        Implements behavior for the built in round() function. 
        n is the number of decimal places to round to.
        """
        cust_self = float(self)
        return round(cust_self, ndigits)

    def __floor__(self):
        """
        Implements behavior for math.floor(), i.e., rounding down to the nearest integer.
        """
        
        cust_self = float(self)
        return self.__class__(math.floor(cust_self))

    def __ceil__(self):
        """
        Implements behavior for math.ceil(), i.e., rounding up to the nearest integer.
        """
        
        cust_self = float(self)
        return self.__class__(math.ceil(cust_self))

    def __trunc__(self):
        """
        Implements behavior for math.trunc(), i.e., truncating to an integral.
        """
        
        return self.__class__(math.trunc(self))

    def __add__(self, other):
        """
        Implements addition.
        """
        
        return self.__class__(float.__add__(self, other))

    def __sub__(self, other):
        """
        Implements subtraction.
        """
        
        return self.__class__(float.__sub__(self, other))

    def __mul__(self, other):
        """
        Implements multiplication.
        """
        return self.__class__(float.__mul__(self, other))

    def __floordiv__(self, other):
        """
        Implements integer division using the // operator.
        """
       
        return self.__class__(float.__floordiv__(self, other))

    def __div__(self, other):
        """
        Implements division using the / operator.
        """
        return self.__class__(float.__div__(self, other))

    def __truediv__(self, other):
        """
        Implements true division. Note that this only works when 
        from __future__ #import division is in effect.
        """
        return self.__class__(float.__truediv__(self, other))

    def __mod__(self, other):
        """
        Implements modulo using the % operator.
        """
        return self.__class__(float.__mod__(self, other))

    def __divmod__(self, other):
        """
        Implements behavior for long division using the divmod() built in function.
        """
        div, mod = float.__divmod__(self, other)
        return self.__class__(div), self.__class__(mod)

    def __pow__(self, other):
        """
        Implements behavior for exponents using the ** operator.
        """
        return self.__class__(float.__pow__(self, other))

    def __radd__(self, other):
        """
        Implements reflected addition.
        """
        return self.__class__(float.__radd__(self, other))

    def __rsub__(self, other):
        """
        Implements reflected subtraction.
        """
        other = float(str(other))
        return self.__class__(float.__rsub__(self, other))

    def __rmul__(self, other):
        """
        Implements reflected multiplication.
        """
        try:
            other = float(str(other))
            return self.__class__(float.__rmul__(self, other))
        except ValueError:
            raise TypeError("can't multiply sequence by non-int of type 'float'")

    def __rfloordiv__(self, other):
        """
        Implements reflected integer division using the // operator.
        """
        return self.__class__(float.__rfloordiv__(self, other))

    def __rdiv__(self, other):
        """
        Implements reflected division using the / operator.
        """
        return self.__class__(float.__rdiv__(self, other))

    def __rtruediv__(self, other):
        """
        Implements reflected true division. Note that this only works when 
        from __future__ #import division is in effect.
        """
        return self.__class__(float.__rtruediv__(self, other))

    def __rmod__(self, other):
        """
        Implements reflected modulo using the % operator.
        """
        return self.__class__(float.__rmod__(self, other))

    def __rdivmod__(self, other):
        """
        Implements behavior for long division using the divmod() built in function, 
        when divmod(other, self) is called.
        """
        div, mod = float.__rdivmod__(self, other)
        return self.__class__(div), self.__class__(mod)

    def __rpow__(self, other):
        """
        Implements behavior for reflected exponents using the ** operator.
        """
        return self.__class__(float.__rpow__(self, other))

    def __index__(self):
        """
        When you are using a trimfloat as an index, it needs to act like an int.
        """
        return int(self)

'''
class trimint
'''

class trimint(int):
    def __str__(self):
        """
        Format a float so it prints like PHP and JavaScript, with no trailing zeroes.
        """
        return ('%f' % self).rstrip('0').rstrip('.')

    def __repr__(self):
        """
        Format a float so it prints like PHP and JavaScript, with no trailing zeroes.
        """
        return ('%f' % self).rstrip('0').rstrip('.')

    def __pos__(self):
        """
        Implements behavior for unary positive (e.g. +some_object)
        """

        return self.__class__(int.__pos__(self))

    def __neg__(self):
        """
        Implements behavior for negation (e.g. -some_object)
        """

        return self.__class__(int.__neg__(self))

    def __abs__(self):
        """
        Implements behavior for the built in abs() function.
        """

        return self.__class__(int.__abs__(self))

    def __add__(self, other):
        """
        Implements addition.
        """

        result = int.__add__(self, other)
        return result if result is NotImplemented else self.__class__(result)

    def __sub__(self, other):
        """
        Implements subtraction.
        """

        result = int.__sub__(self, other)
        return result if result is NotImplemented else self.__class__(result)

    def __mul__(self, other):
        """
        Implements multiplication.
        """
        result = int.__mul__(self, other)
        return result if result is NotImplemented else self.__class__(result)

    def __floordiv__(self, other):
        """
        Implements integer division using the // operator.
        """

        return trimfloat.__floordiv__(trimfloat(self), other)

    def __div__(self, other):
        """
        Implements division using the / operator.
        """
        return trimfloat.__div__(trimfloat(self), other)

    def __truediv__(self, other):
        """
        Implements true division. Note that this only works when
        from __future__ #import division is in effect.
        """
        return trimfloat.__truediv__(trimfloat(self), other)

    def __mod__(self, other):
        """
        Implements modulo using the % operator.
        """
        result = int.__mod__(self, other)
        return result if result is NotImplemented else self.__class__(result)

    def __divmod__(self, other):
        """
        Implements behavior for long division using the divmod() built in function.
        """
        return trimfloat.__divmod__(trimfloat(self), other)

    def __pow__(self, other):
        """
        Implements behavior for exponents using the ** operator.
        """
        result = int.__pow__(self, other)
        return result if result is NotImplemented else self.__class__(result)

    def __radd__(self, other):
        """
        Implements reflected addition.
        """
        result = int.__radd__(self, other)
        return result if result is NotImplemented else self.__class__(result)

    def __rsub__(self, other):
        """
        Implements reflected subtraction.
        """
        result = int.__rsub__(self, other)
        return result if result is NotImplemented else self.__class__(result)

    def __rmul__(self, other):
        """
        Implements reflected multiplication.
        """
        #return trimfloat.__rmul__(trimfloat(self), other)
        try:
            other = int(str(other))
            return self.__class__(int.__rmul__(self, other))
        except ValueError:
            return super(trimint, self).__rmul__(other)

    def __rfloordiv__(self, other):
        """
        Implements reflected integer division using the // operator.
        """
        return trimfloat.__rfloordiv__(trimfloat(self), other)

    def __rdiv__(self, other):
        """
        Implements reflected division using the / operator.
        """
        return trimfloat.__rdiv__(trimfloat(self), other)

    def __rtruediv__(self, other):
        """
        Implements reflected true division. Note that this only works when
        from __future__ #import division is in effect.
        """
        return trimfloat.__rtruediv__(trimfloat(self), other)

    def __rmod__(self, other):
        """
        Implements reflected modulo using the % operator.
        """
        result = int.__rmod__(self, other)
        return result if result is NotImplemented else self.__class__(result)

    def __rdivmod__(self, other):
        """
        Implements behavior for long division using the divmod() built in function,
        when divmod(other, self) is called.
        """
        return trimfloat.__rdivmod__(trimfloat(self), other)

    def __rpow__(self, other):
        """
        Implements behavior for reflected exponents using the ** operator.
        """
        result = int.__rpow__(self, other)
        return result if result is NotImplemented else self.__class__(result)

